<?php

namespace Drupal\healthcheck;
use Drupal\Component\Discovery\YamlDirectoryDiscovery;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\healthcheck\Finding\Finding;
use Drupal\healthcheck\Finding\FindingStatus;
use Drupal\healthcheck\Plugin\HealthcheckPluginManager;

/**
 * Class FindingService.
 */
class FindingService implements FindingServiceInterface {

  use StringTranslationTrait;

  /**
   * Drupal\Core\Extension\ModuleHandlerInterface definition.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;
  /**
   * Drupal\Core\Extension\ThemeHandlerInterface definition.
   *
   * @var \Drupal\Core\Extension\ThemeHandlerInterface
   */
  protected $themeHandler;

  /**
   * The HealthcheckPluginManager
   *
   * @var HealthcheckPluginManager;
   */
  protected $healthcheck_plugin_mgr;

  /**
   * The YAML directory discovery.
   *
   * @var null|YamlDirectoryDiscovery
   */
  protected $discovery = NULL;

  /**
   * Constructs a new FindingService object.
   */
  public function __construct(ModuleHandlerInterface $module_handler, ThemeHandlerInterface $theme_handler, HealthcheckPluginManager $healthcheck_plugin_mgr) {
    $this->moduleHandler = $module_handler;
    $this->themeHandler = $theme_handler;
    $this->healthcheck_plugin_mgr = $healthcheck_plugin_mgr;
  }

  /**
   * {@inheritdoc}
   */
  public function build($check, $key, $status, $data = []) {
    // Create a stub finding with the given parameters.
    $finding = new Finding($status, $check, $key, '', $data);

    // Try to load the label and message from YAML files.
    $findings = $this->getFindings();
    if (empty($findings[$key])) {
      return $finding;
    }

    $finding_info = $findings[$key];

    $statuses = FindingStatus::getTextConstants();
    $status_key = $statuses[$status];

    if (!empty($finding_info[$status_key]['label'])) {
      $label = $finding_info[$status_key]['label'];

      $finding->setLabel($this->t($label, $this->labelify($data)));
    }

    if (!empty($finding_info[$status_key]['message'])) {
      $message = $finding_info[$status_key]['message'];
      
      $finding->setMessage($this->t($message, $this->labelify($data)));
    }

    return $finding;
  }

  protected function getFindings() {
    $findings = [];

    // Check if we have cached findings.
    $cache = \Drupal::cache()->get('healthcheck.findings');

    // No? Load 'em up.
    if (empty($cache)) {
      // Discover all of the YAML files.
      $discovery = $this->getDiscovery();

      // Parse them all.
      $files = $discovery->findAll();

      // Merge the found files together keyed by ID.
      foreach ($files as $module => $items) {
        foreach ($items as $item) {
          $id = $item['id'];

          if (empty($findings[$id])) {
            $findings[$id] = $item;
          }
        }
      }

      // Allow modules to alter them.
      $this->moduleHandler->alter('healthcheck_findings', $findings);

      // Save the result to cache.
      \Drupal::cache()->set('healthcheck.findings', $findings, CacheBackendInterface::CACHE_PERMANENT);
    }
    else {
      // Cache get? Get the files.
      $findings = $cache->data;
    }

    return $findings;
  }

  /**
   * Discover any YAML directories used for healthcheck findings.
   *
   * @return \Drupal\Component\Discovery\YamlDirectoryDiscovery
   *   A new YamlDirectoryDiscovery for healthcheck findings.
   */
  protected function getDiscovery() {
    // This was based on a 8.7 issue on help_topics.
    // @see https://www.drupal.org/project/drupal/issues/2920309

    if (empty($this->discovery)) {
      $directories = array_merge($this->moduleHandler->getModuleDirectories(), $this->themeHandler->getThemeDirectories());

      $directories = array_map(function ($dir) {
        return [$dir . '/healthcheck_finding'];
      }, $directories);

      $file_cache_key_suffix = 'healthcheck_finding';
      $id_key = 'id';
      $this->discovery = new YamlDirectoryDiscovery($directories, $file_cache_key_suffix, $id_key);
    }

    return $this->discovery;
  }

  /**
   * Labelifies the data array for use as text placeholders
   *
   * @param $data
   *   The data array to labelify.
   *
   * @return array
   *   The labelified array
   */
  protected function labelify($data) {
    $out = [];

    // If the data isn't an array, return it.
    if (!is_array($data)) {
      return $data;
    }

    foreach ($data as $key => $value) {
      $label_key = ':' . $key;
      $out[$label_key] = $value;
    }

    return $out;
  }
}
